# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identity/ClaimsIdentityRpc.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from azure_functions_worker.protos.shared import NullableTypes_pb2 as shared_dot_NullableTypes__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='identity/ClaimsIdentityRpc.proto',
  package='',
  syntax='proto3',
  serialized_options=_b('\n*com.microsoft.azure.functions.rpc.messages'),
  serialized_pb=_b('\n identity/ClaimsIdentityRpc.proto\x1a\x1ashared/NullableTypes.proto\"\xb0\x01\n\x11RpcClaimsIdentity\x12,\n\x13\x61uthentication_type\x18\x01 \x01(\x0b\x32\x0f.NullableString\x12(\n\x0fname_claim_type\x18\x02 \x01(\x0b\x32\x0f.NullableString\x12(\n\x0frole_claim_type\x18\x03 \x01(\x0b\x32\x0f.NullableString\x12\x19\n\x06\x63laims\x18\x04 \x03(\x0b\x32\t.RpcClaim\"\'\n\x08RpcClaim\x12\r\n\x05value\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\tB,\n*com.microsoft.azure.functions.rpc.messagesb\x06proto3')
  ,
  dependencies=[shared_dot_NullableTypes__pb2.DESCRIPTOR,])




_RPCCLAIMSIDENTITY = _descriptor.Descriptor(
  name='RpcClaimsIdentity',
  full_name='RpcClaimsIdentity',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='authentication_type', full_name='RpcClaimsIdentity.authentication_type', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name_claim_type', full_name='RpcClaimsIdentity.name_claim_type', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='role_claim_type', full_name='RpcClaimsIdentity.role_claim_type', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='claims', full_name='RpcClaimsIdentity.claims', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=241,
)


_RPCCLAIM = _descriptor.Descriptor(
  name='RpcClaim',
  full_name='RpcClaim',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='RpcClaim.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='RpcClaim.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=243,
  serialized_end=282,
)

_RPCCLAIMSIDENTITY.fields_by_name['authentication_type'].message_type = shared_dot_NullableTypes__pb2._NULLABLESTRING
_RPCCLAIMSIDENTITY.fields_by_name['name_claim_type'].message_type = shared_dot_NullableTypes__pb2._NULLABLESTRING
_RPCCLAIMSIDENTITY.fields_by_name['role_claim_type'].message_type = shared_dot_NullableTypes__pb2._NULLABLESTRING
_RPCCLAIMSIDENTITY.fields_by_name['claims'].message_type = _RPCCLAIM
DESCRIPTOR.message_types_by_name['RpcClaimsIdentity'] = _RPCCLAIMSIDENTITY
DESCRIPTOR.message_types_by_name['RpcClaim'] = _RPCCLAIM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RpcClaimsIdentity = _reflection.GeneratedProtocolMessageType('RpcClaimsIdentity', (_message.Message,), {
  'DESCRIPTOR' : _RPCCLAIMSIDENTITY,
  '__module__' : 'identity.ClaimsIdentityRpc_pb2'
  # @@protoc_insertion_point(class_scope:RpcClaimsIdentity)
  })
_sym_db.RegisterMessage(RpcClaimsIdentity)

RpcClaim = _reflection.GeneratedProtocolMessageType('RpcClaim', (_message.Message,), {
  'DESCRIPTOR' : _RPCCLAIM,
  '__module__' : 'identity.ClaimsIdentityRpc_pb2'
  # @@protoc_insertion_point(class_scope:RpcClaim)
  })
_sym_db.RegisterMessage(RpcClaim)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
